@@ -1,7 +1,7 @@
 plot3D <- function(x,...) UseMethod("plot3D")
 
 plot3D.rmult <- function(x,parts=1:3,...,center=FALSE,scale=FALSE,add=FALSE,axes=!add,cex=2,vlabs=colnames(x),size=cex,bbox=FALSE,col=1) {
-  require("rgl")
+  requireNamespace("rgl")
   X<-x
   if( ! add ) 
       gsi.reset3D()
@@ -16,19 +16,19 @@ plot3D.rmult <- function(x,parts=1:3,...,center=FALSE,scale=FALSE,add=FALSE,axes
       scale <- diag(scale,1)
     }
     scale[4,4] <- max(diag(scale)[-4]) 
-    points3d(x[,1],x[,2],x[,3],size=size,...,col=col)
-    par3d(userMatrix=scale)
+    rgl::points3d(x[,1],x[,2],x[,3],size=size,...,col=col)
+    rgl::par3d(userMatrix=scale)
     gsi.filtercall("axis3D")
                                         #  axis3D(axis.origin=axis.origin,axis.scale=axis.scale,
                                         #         axis.col=axis.col,vlabs=vlabs,
                                         #         vlabs.col=vlabs.col,bbox=bbox,
                                         #         axis.lwd=axis.lwd,axis.len=axis.len,
                                         #         axis.angle=axis.angle,oth=orth,axes=axes)
-    invisible(NULL)
+    invisible(rmult(x))
   }
 
 gsi.reset3D <- function(userMatrix=diag(rep(1,4))) {
-  require("rgl")
+  requireNamespace("rgl")
   rgl::clear3d()
   rgl::par3d(userMatrix=userMatrix)
 }
@@ -37,7 +37,7 @@ gsi.reset3D <- function(userMatrix=diag(rep(1,4))) {
 gsi.filtercall <- function(fkt,...,overwrite=NULL,transmit=character(0),default=list(...),prefix=if(is.character(fkt)) fkt else NULL) {
   #oo <- function(x) {cat(deparse(substitute(x)),"=\n");print(x)}
   #oo <- function(x) {}
-  require("rgl")
+  requireNamespace("rgl")
   prefix
   if(is.character(fkt)) fkt <- get(fkt,mode="function")
   #oo(fkt)
@@ -80,22 +80,23 @@ gsi.filtercall <- function(fkt,...,overwrite=NULL,transmit=character(0),default=
 }
 
 plot3D.default <- function(x,...,add=FALSE,bbox=TRUE,axes=FALSE,cex=1,size=cex,col=1) {
-  require("rgl")
+  requireNamespace("rgl")
   X<-x
   if( ! add )
     gsi.reset3D()
   radius <- max(max(X[,1])-min(X[,1]),max(X[,2])-min(X[,2]),max(X[,3])-min(X[,3]))/100*cex
   rgl::spheres3d(X[,1],X[,2],X[,3],...,radius=radius,col=col)
-  if( bbox) rgl.bbox()
+  if( bbox) rgl::rgl.bbox()
   if( any(axes) ) arrows3D(diag(c(0,0,0)),diag(c(1,1,1))) 
-  invisible(NULL)
+  invisible(X[,1:3])
 }
 
 
 
 plot3D.acomp <- function(x,parts=1:min(ncol(X),4),...,lwd=2,axis.col="gray",add=FALSE,cex=2,vlabs=colnames(x),vlabs.col=axis.col,center=FALSE,scale=FALSE,log=FALSE,bbox=FALSE,axes=TRUE,size=cex,col=1) {
-  require("rgl")
+  requireNamespace("rgl")
   X<-x
+  out = NULL
   if( length(parts) == 3 ) {
    if( log ) {
      x <- clr(scale(acomp(X,parts=parts),center=center,scale=scale))
@@ -104,7 +105,8 @@ plot3D.acomp <- function(x,parts=1:min(ncol(X),4),...,lwd=2,axis.col="gray",add=
        if( axes )
          arrows3D(diag(c(0,0,0)),diag(c(1,1,1)),labs=vlabs,col=axis.col)
      }
-      points3d(x[,1],x[,2],x[,3],size=size,...,col=col)
+      rgl::points3d(x[,1],x[,2],x[,3],size=size,...,col=col)
+      out = rmult(x[,1:3])
     } else {
       x <- scale(acomp(X,parts=parts),center=center,scale=scale)
       if( ! add ) {
@@ -112,13 +114,14 @@ plot3D.acomp <- function(x,parts=1:min(ncol(X),4),...,lwd=2,axis.col="gray",add=
         corners <- rbind(diag(rep(1,3)),c(0,0,0))
         cl <- corners[c(1,2,3,4,1,3,2,4),]
         if( axes ) 
-          lines3d(cl[,1],cl[,2],cl[,3],col=axis.col,size=lwd)
+          rgl::lines3d(cl[,1],cl[,2],cl[,3],col=axis.col,size=lwd)
         if( !is.null(vlabs) )
-          texts3d(corners[,1],corners[,2],corners[,3],c(vlabs,"0"),col=vlabs.col)
+          rgl::texts3d(corners[,1],corners[,2],corners[,3],c(vlabs,"0"),col=vlabs.col)
       }
-      points3d(x[,1],x[,2],x[,3],size=size,...,col=col)
+      rgl::points3d(x[,1],x[,2],x[,3],size=size,...,col=col)
+      out = rmult(x[,1:3])
     }
-   rgl.viewpoint(45,35.4)
+   rgl::rgl.viewpoint(45,35.4)
  } else if( length(parts)==4 ) {
    x <- clo(X,parts=parts)
    if( log ) {
@@ -129,7 +132,8 @@ plot3D.acomp <- function(x,parts=1:min(ncol(X),4),...,lwd=2,axis.col="gray",add=
          arrows3D(corners*0,corners,col=axis.col,size=lwd,labs=vlabs)
      }
      ilrx <- ilr(scale(acomp(x),center=center,scale=scale))
-     points3d(ilrx[,1],ilrx[,2],ilrx[,3],size=size,...,col=col)
+     rgl::points3d(ilrx[,1],ilrx[,2],ilrx[,3],size=size,...,col=col)
+     out = rmult(ilrx[,1:3])
    } else {
      if( ! add ) {
        gsi.reset3D()
@@ -137,20 +141,21 @@ plot3D.acomp <- function(x,parts=1:min(ncol(X),4),...,lwd=2,axis.col="gray",add=
        cornerlines <- corners[c(1,2,3,4,1,3,2,4),]
        cl <- ipt(cornerlines)
        if( axes )
-         lines3d(cl[,1],cl[,2],cl[,3],col=axis.col,size=lwd)
+         rgl::lines3d(cl[,1],cl[,2],cl[,3],col=axis.col,size=lwd)
      }
      iptx <- ipt(scale(acomp(x),center=center,scale=scale))
-     points3d(iptx[,1],iptx[,2],iptx[,3],size=size,...,col=col)
+     rgl::points3d(iptx[,1],iptx[,2],iptx[,3],size=size,...,col=col)
+     out = rmult(iptx[,1:3])
      if( !is.null(vlabs) ) {
        cc <- ipt(corners)
-       texts3d(cc[,1],cc[,2],cc[,3],c(vlabs),col=vlabs.col)
+       rgl::texts3d(cc[,1],cc[,2],cc[,3],c(vlabs),col=vlabs.col)
      }
    }
  } else
   stop("Wrong number of parts")
   if( bbox )
-    bbox3d()
-  invisible(NULL)
+    rgl::bbox3d()
+  invisible(out)
 }
   
 plot3D.rcomp <- plot3D.acomp
@@ -158,6 +163,7 @@ plot3D.rcomp <- plot3D.acomp
 
 
 axis3D <- function(axis.origin=c(0,0,0),axis.scale=1,axis.col="gray",vlabs=c("x","y","z"),vlabs.col=axis.col,bbox=FALSE,axis.lwd=2,axis.len=mean(axis.scale)/10,axis.angle=30,orth=c(1,0.0001,0.000001),axes=TRUE,...) {
+  requireNamespace("rgl")
   if( axes ) {
     M <- rbind(axis.origin,axis.origin,axis.origin)
     if( length(axis.scale)== 1) axis.scale <- rep(axis.scale,3)
@@ -166,28 +172,28 @@ axis3D <- function(axis.origin=c(0,0,0),axis.scale=1,axis.col="gray",vlabs=c("x"
              code=2,col=axis.col,orth=orth,labs=vlabs,...)
   }
   if( bbox ) {
-    bbox3d(xlab=vlabs[1],ylab=vlabs[2],zlab=vlabs[3])
+    rgl::bbox3d(xlab=vlabs[1],ylab=vlabs[2],zlab=vlabs[3])
   }
 }
 
 plot3D.rplus <- function(x,parts=1:3,...,vlabs=NULL,add=FALSE,bbox=FALSE,cex=1,size=cex,axes=TRUE,col=1) {
-  require("rgl")
+  requireNamespace("rgl")
   X<-x
   if(! add )
     gsi.reset3D()
   XX <- oneOrDataset(iit(rplus(X,parts=parts)))
-  plot3D(rmult(XX),...,bbox=bbox,col=col)
+  out = plot3D(rmult(XX),...,bbox=bbox,col=col)
   if( missing(vlabs) )
     vlabs <- colnames(XX)
   if( axes ) arrows3D(diag(c(0,0,0)),diag(mean(rplus(XX))),labs=vlabs)
-  invisible(NULL)
+  invisible(out)
 }
 
 plot3D.aplus <- function(x,parts=1:3,...,vlabs=NULL,add=FALSE,log=TRUE,bbox=FALSE,axes=TRUE,col=1) {
-  require("rgl")
+  requireNamespace("rgl")
   X<-x
   if( ! log ) {
-    plot3D(rplus(X),parts=parts,...,col=col,vlabs=vlabs,add=add,bbox=bbox)
+    out = plot3D(rplus(X),parts=parts,...,col=col,vlabs=vlabs,add=add,bbox=bbox)
     return()
   }
   if(! add )
@@ -195,12 +201,12 @@ plot3D.aplus <- function(x,parts=1:3,...,vlabs=NULL,add=FALSE,log=TRUE,bbox=FALS
   XX <- oneOrDataset(ilt(aplus(X,parts=parts)))
   if( missing(vlabs) )
     vlabs <- colnames(XX)
-  plot3D(rmult(XX),...,col=col,bbox=bbox)
+  out = plot3D(rmult(XX),...,col=col,bbox=bbox)
   mm <- mean(rmult(XX))
   if( axes )
     arrows3D(diag(mm+c(0,0,0)),diag(mm+c(1,1,1)),labs=vlabs)
  # bbox3d()
-  invisible(NULL)
+  invisible(out)
 }
 
 
@@ -209,14 +215,16 @@ arrows3D <- function(...) UseMethod("arrows3D")
 #par("fg") and par("col") # when the plotting functions put their reference arrows, the call to par() generates a blank plot, replaced by col=1
 arrows3D.default <- function(x0,x1,...,length=0.25,
                      angle=30,code=2,col="black",lty=NULL,lwd=2,orth=c(1,0.0001,0.0000001),labs=NULL,size=lwd) {
+  requireNamespace("rgl")
   X <- oneOrDataset(x0,x1)
   Y <- oneOrDataset(x1,x0)
   if( ! is.null(labs))
-    texts3d(Y[,1],Y[,2],Y[,3],labs,...,col=col)
+    rgl::texts3d(Y[,1],Y[,2],Y[,3],labs,...,col=col)
   XX <- rbind(X,Y)
   XX[(1:nrow(X)-1)*2+1,]<- X
   XX[(1:nrow(X)-1)*2+2,]<- Y
-  segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
+  rgl::segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
+  out = rmult(XX[,1:3])
   if( code > 0 ) {
     XY <- rmult(X)-rmult(Y)
     XL <- norm(XY)
@@ -229,21 +237,21 @@ arrows3D.default <- function(x0,x1,...,length=0.25,
     if( code  %% 2 == 1 ) {
       XX[(1:nrow(X)-1)*2+1,]<- X
       XX[(1:nrow(X)-1)*2+2,]<- X - c*length*XY + ZD*(XL*length*s)
-      segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
+      rgl::segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
       XX[(1:nrow(X)-1)*2+1,]<- X
       XX[(1:nrow(X)-1)*2+2,]<- X - c*length*XY - ZD*(XL*length*s)
-      segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
+      rgl::segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
     }
     if( floor(code/2)  %% 2 == 1 ) {
       XX[(1:nrow(X)-1)*2+1,]<- Y
       XX[(1:nrow(X)-1)*2+2,]<- Y + c*length*XY + ZD*(XL*length*s)
-      segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
+      rgl::segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
       XX[(1:nrow(X)-1)*2+1,]<- Y
       XX[(1:nrow(X)-1)*2+2,]<- Y + c*length*XY - ZD*(XL*length*s)
-      segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
+      rgl::segments3d(XX[,1],XX[,2],XX[,3],col=col,size=size,...)
     }
   }
-  invisible(NULL)
+  invisible(out)
 }
 
 biplot3D <- function(x,...) UseMethod("biplot3D")
@@ -254,14 +262,14 @@ biplot3D.default <- function(x,y,var.axes=TRUE,
 #            xlim  = NULL, ylim  = NULL, 
 #            main = NULL, sub = NULL, xlab = NULL, ylab = NULL,
                              ...,add=FALSE){
-  require("rgl")
+  requireNamespace("rgl")
   if( !add ) {
     gsi.reset3D();
   }
   x <- oneOrDataset(x)
-  points3d(x[,1],x[,2],x[,3],size=cex[1],col=col[[1]],...)
+  rgl::points3d(x[,1],x[,2],x[,3],size=cex[1],col=col[[1]],...)
   if( ! is.null(xlabs) ) {
-    texts3d(x[,1],x[,2],x[,3],xlabs,...)
+    rgl::texts3d(x[,1],x[,2],x[,3],xlabs,...)
   }
   if( var.axes ) {
     y <- oneOrDataset(expand*y)
@@ -271,18 +279,19 @@ biplot3D.default <- function(x,y,var.axes=TRUE,
     
   } else {
     y <- oneOrDataset(expand*y)
-    points3d(y[,1],y[,2],y[,3],
+    rgl::points3d(y[,1],y[,2],y[,3],
              size=cex[[1%%length(cex)+1]],
              col=col[[1%%length(col)+1]],...)
   }
+  out = rmult(y[,1:3])
   if( ! is.null(ylabs) ) {
-    texts3d(y[,1],y[,2],y[,3],ylabs,...)
+    rgl::texts3d(y[,1],y[,2],y[,3],ylabs,...)
   }
-  invisible(NULL)
+  invisible(out)
 }
 
 biplot3D.princomp <- function(x,choices=1:3,scale=1,...,comp.col=1,comp.labs=paste("Comp.",1:3),scale.scores=lambda[choices]^(1-scale),scale.var=scale.comp,scale.comp=sqrt(lambda[choices]),scale.disp=1/scale.comp) {
-  require("rgl")
+  requireNamespace("rgl")
   lambda <- x$sdev^2
   if( length(scale.var) == 1 ) scale.var <- rep(scale.var,3)
   if( length(scale.scores) == 1 ) scale.scores <-  rep(scale.scores,3)
@@ -290,12 +299,12 @@ biplot3D.princomp <- function(x,choices=1:3,scale=1,...,comp.col=1,comp.labs=pas
   if( length(scale.disp) == 1 ) scale.disp <-  rep(scale.disp,3)
 #  biplot3D.default( unclass(x$scores[,choices]) %*% diag(lambda[choices]^(1-scale)),
 #  unclass(x$loadings[,choices])%*% diag(lambda[choices]^scale),...)
-  biplot3D.default( unclass(x$scores[,choices]) %*% diag(scale.scores),
+  out = biplot3D.default( unclass(x$scores[,choices]) %*% diag(scale.scores),
   unclass(x$loadings[,choices])%*% diag(scale.var),...,ylabs=rownames(x$loadings),xlabs=rownames(x$scores))#
   comp <- diag(scale.comp)
   arrows3D(c(0,0,0),comp,col=comp.col)
   if( ! is.null(comp.labs) )
-    texts3d(comp[,1],comp[,2],comp[,3],comp.labs,col=comp.col)
-  par3d(userMatrix=diag(c(scale.disp/max(scale.disp),1)))
-  invisible(NULL)
+    rgl::texts3d(comp[,1],comp[,2],comp[,3],comp.labs,col=comp.col)
+  rgl::par3d(userMatrix=diag(c(scale.disp/max(scale.disp),1)))
+  invisible(out)
 }
